% GHF_SmithChart  Plots a Smith Chart
% Formulae: http://www.maxim-ic.com/appnotes.cfm/appnote_number/742/
% Check: http://www.volker-lotze.de/content/smithtool/smithtool.php
% Notation: GHF_Skript, S.83
% Usage:		GHF_SmithChart(x,y)
%		where x is for an Impedance Smith Chart
%		and y is for an Admittance Smith Chart.
%		Plots a smith chart.
% Example:      GHF_SmithChart(0,1)
%		Draws an Admittance Smith Chart
%			GHF_SmithChart(1,0)
%		Draws an Impedance Smith Chart
% Author:       Olexandr Lednytskyy - (05-06-2008)

function GHF_SmithChart(Imp,Adm)
	if nargin<1
		error('GHF_SmithChart.m: One input argument (1 or 0) for a Impedance Smith Chart as first parameter and one for a Admittance Smith Chart as second parameter required.');
	elseif ((Imp==1 || Imp==true) && (Adm==0 || Adm==false))
		constructImpedanceSmith=1;
		constructAdmittanceSmith=0;
	elseif ((Imp==0 || Imp==false) && (Adm==1 || Adm==true))
		constructImpedanceSmith=0;
		constructAdmittanceSmith=1;
	else
		constructImpedanceSmith=1;
		constructAdmittanceSmith=1;
	end

%{
constructImpedanceSmith=true;
constructAdmittanceSmith=false;
%}

%figure('Position',[200 40 900 900]); % SmithChart in a separate window
% CHART OPTIONS, feel free to change the number of array elements :-)
PlotRho=0.5; % Radien of concentric Circles around (0;0)
plot(0,0,'k*'); hold on
PlotReZ=[0.2 0.5 1.0 2.2]; % Radii for R=const in the r-plain
PlotReY=PlotReZ; % Radien bei G=const in der r-Ebene
PlotImZpos=[0.1 0.35 1.0 2.0]; % Radii for X=const in the r-plain
PlotImZneg=-PlotImZpos;
PlotImYpos=[0.1 0.35 1.0 2.0]; % Radii for B=const in the r-plain
PlotImYneg=-PlotImYpos;

% Circles of Rho
NOP=500; % Number of points
ThetaRadCirRho=linspace(0,2*pi,NOP);
RhoNoIndexImp(length(PlotRho))=0;
RhoNoIndexAdm(length(PlotRho))=0;
for trcRho=1:length(ThetaRadCirRho)
	for indexPlotRho=1:length(PlotRho)
		% RhoCircle. Points G=x+jy
		Rho(trcRho)=exp(j*ThetaRadCirRho(trcRho));
		RhoNoIndex(indexPlotRho,trcRho)=PlotRho(indexPlotRho)*exp(j*ThetaRadCirRho(trcRho));
		if trcRho==NOP % for Labels of the Ordinate-Axis for the Impedance Smith Chart
			RhoNoIndexImp(indexPlotRho)=RhoNoIndex(indexPlotRho,trcRho);
		end
		if trcRho==NOP/2 % for Labels of the Ordinate-Axis for the Admittance Smith Chart
			RhoNoIndexAdm(indexPlotRho)=RhoNoIndex(indexPlotRho,trcRho);
		end
	end
end
colorCRho=[0,0,0];
plot(real(Rho),imag(Rho),'Color',colorCRho) % border
hold on
for indexPlotRho=1:length(PlotRho)
	plot(real(RhoNoIndex(indexPlotRho,:)),imag(RhoNoIndex(indexPlotRho,:)),'Color',colorCRho)
	hold on
	if indexPlotRho==1
		reRhoValuesImp=real(RhoNoIndexImp(indexPlotRho));
		imRhoValuesImp=imag(RhoNoIndexImp(indexPlotRho));
		reRhoValuesAdm=real(RhoNoIndexAdm(indexPlotRho));
		imRhoValuesAdm=imag(RhoNoIndexAdm(indexPlotRho));
	else
		reRhoValuesImp=horzcat(reRhoValuesImp,real(RhoNoIndexImp(indexPlotRho)));
		imRhoValuesImp=horzcat(imRhoValuesImp,imag(RhoNoIndexImp(indexPlotRho)));
		reRhoValuesAdm=horzcat(reRhoValuesAdm,real(RhoNoIndexAdm(indexPlotRho)));
		imRhoValuesAdm=horzcat(imRhoValuesAdm,imag(RhoNoIndexAdm(indexPlotRho)));
	end
	if ((constructImpedanceSmith==true) || (constructImpedanceSmith==true && constructAdmittanceSmith==true))
		imRhoLabelsImp=text(reRhoValuesImp(indexPlotRho),imRhoValuesImp(indexPlotRho),...
			[num2str(PlotRho(indexPlotRho),'%10.2f'),'  '],...
			'FontSize',9,'color',colorCRho);
		set(imRhoLabelsImp,'Rotation',-90);
	else % constructAdmittanceSmith==true
		imRhoLabelsAdm=text(reRhoValuesAdm(indexPlotRho),imRhoValuesAdm(indexPlotRho),...
			[num2str(PlotRho(indexPlotRho),'%10.2f'),'  '],...
			'FontSize',9,'color',colorCRho);
		set(imRhoLabelsAdm,'Rotation',90);
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if constructImpedanceSmith==true
% Circles of Real Z=j*ZL for various values of load resistance, R=const
ThetaRadCirReZ=linspace(0,2*pi,NOP);
ZNoIndexLabel(length(PlotReZ))=0;
for trcReZ=1:length(ThetaRadCirReZ)
	for indexPlotReZ=1:length(PlotReZ)
		r=PlotReZ(indexPlotReZ); % Diagramm-Radius, Reflexionsfaktor
		% Real Circle. Points G=x + jy
		ZNoIndex(indexPlotReZ,trcReZ)=(1/(r+1))*(r+exp(j*ThetaRadCirReZ(trcReZ)));
		if trcReZ==round(NOP/2) % for Labels
			ZNoIndexLabel(indexPlotReZ)=ZNoIndex(indexPlotReZ,trcReZ);
		end
	end
end
colorCReZ=[.2,.6,.4];
for indexPlotReZ=1:length(PlotReZ)
	plot(real(ZNoIndex(indexPlotReZ,:)),imag(ZNoIndex(indexPlotReZ,:)),'Color',colorCReZ)
	hold on
	if indexPlotReZ==1
		reZNoValues=real(ZNoIndexLabel(indexPlotReZ));
		imZNoValues=imag(ZNoIndexLabel(indexPlotReZ));
	else
		reZNoValues=horzcat(reZNoValues,real(ZNoIndexLabel(indexPlotReZ)));
		imZNoValues=horzcat(imZNoValues,imag(ZNoIndexLabel(indexPlotReZ)));
	end
	reZLabels=text(reZNoValues(indexPlotReZ),imZNoValues(indexPlotReZ),...
		[num2str(PlotReZ(indexPlotReZ),'%10.2f'),'  '],...
		'FontSize',9,'color',colorCReZ);
	set(reZLabels,'Rotation',90);
end

exakt=1e-2;
plotCounter_pos=0; % zeichne nur eine von gefundenen L?sungen
plotCounter_neg=0; % zeichne nur eine von gefundenen L?sungen
% Circles of Imaginary Z for Values of constant imaginary load impedances, X=const
ThetaRadCirImZ=linspace(0,2*pi,NOP*4);
ZNoIndexPosLabel(length(PlotImZpos))=0;
ZNoIndexNegLabel(length(PlotImZpos))=0;
%{
for trcRho=1:length(ThetaRadCirRho)
	Rho(trcRho)=exp(j*ThetaRadCirRho(trcRho));
end
%}
for trcImZ=1:length(ThetaRadCirImZ)
	for indexPlotImZ=1:length(PlotImZpos)
		rp=PlotImZpos(indexPlotImZ); % Diagramm-Radius, Reflexionsfaktor
		rn=PlotImZneg(indexPlotImZ);
		% Imaginary Circle. Points G=x+jy
		ZNoIndexPos(indexPlotImZ,trcImZ)=(1+j*(1/rp))+(1/rp)*exp(j*ThetaRadCirImZ(trcImZ));
		ZNoIndexNeg(indexPlotImZ,trcImZ)=(1+j*(1/rn))+(1/rn)*exp(j*ThetaRadCirImZ(trcImZ));
		if trcImZ==round(4*NOP/(1+0.60)) % for Labels
			ZNoIndexPosLabel(indexPlotImZ)=ZNoIndexPos(indexPlotImZ,trcImZ);
		end

%{
		for trcRho=1:length(ThetaRadCirRho)
				if ( (abs(real(Rho(trcRho))-real(ZNoIndexPos(indexPlotImZ,trcImZ)))<exakt) && (abs(imag(Rho(trcRho))-imag(ZNoIndexPos(indexPlotImZ,trcImZ)))<exakt) )
					if plotCounter_pos==1
						indexPlotImZ_pos=indexPlotImZ;
						trcImZ_pos=trcImZ;
					end
					if plotCounter_pos>1
						ZNoIndexPos(indexPlotImZ,trcImZ)=ZNoIndexPos(indexPlotImZ_pos,trcImZ_pos);
					end
					plotCounter_pos=plotCounter_pos+1;
				end
				if ( (abs(real(Rho(trcRho))-real(ZNoIndexNeg(indexPlotImZ,trcImZ)))<exakt) && (abs(imag(Rho(trcRho))-imag(ZNoIndexNeg(indexPlotImZ,trcImZ)))<exakt) )
					if plotCounter_neg==1
						indexPlotImZ_neg=indexPlotImZ;
						trcImZ_neg=trcImZ;
					end
					if plotCounter_neg>1
						ZNoIndexNeg(indexPlotImZ,trcImZ)=ZNoIndexNeg(indexPlotImZ_neg,trcImZ_neg);
					end
					plotCounter_neg=plotCounter_neg+1;
				end
		end
%}

	end
end
colorCImZ=[.4,.8,.2];
for indexPlotImZ=1:length(PlotImZpos)
	plot(real(ZNoIndexPos(indexPlotImZ,:)),imag(ZNoIndexPos(indexPlotImZ,:)),'Color',colorCImZ)
	plot(real(ZNoIndexNeg(indexPlotImZ,:)),imag(ZNoIndexNeg(indexPlotImZ,:)),'Color',colorCImZ)
	hold on
	if indexPlotImZ==1
		reZposValues=real(ZNoIndexPosLabel(indexPlotImZ));
		imZposValues=imag(ZNoIndexPosLabel(indexPlotImZ));
	else
		reZposValues=horzcat(reZposValues,real(ZNoIndexPosLabel(indexPlotImZ)));
		imZposValues=horzcat(imZposValues,imag(ZNoIndexPosLabel(indexPlotImZ)));
	end
	ZposLabels=text(reZposValues(indexPlotImZ),imZposValues(indexPlotImZ),...
		['j',num2str(abs(PlotImZpos(indexPlotImZ)),'%10.2f')],...
		'FontSize',9,'color',colorCImZ); % [sign(PlotImZpos(indexPlotImZ)),'j',
	set(ZposLabels,'Rotation',90);
end
end % constructImpedanceSmith==true

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if constructAdmittanceSmith==true
% Circles of Real Y=1/(j*ZL) for various values of load resistance, G=const
ThetaRadCirReY=linspace(0,2*pi,NOP);
YNoIndexLabel(length(PlotReY))=0;
for trcReY=1:length(ThetaRadCirReY)
	for indexPlotReY=1:length(PlotReY)
		r=PlotReY(indexPlotReY); % Diagramm-Radius, Reflexionsfaktor
		% we keep the location of the reflection coefficient fixed and rotate the entire Smith Chart by 180 degree: exp(j*pi)
		YNoIndex(indexPlotReY,trcReY)=exp(j*pi)*(1/(r+1))*(r+exp(j*ThetaRadCirReY(trcReY)));
		if trcReY==round(NOP/2) % for Labels
			YNoIndexLabel(indexPlotReY)=YNoIndex(indexPlotReY,trcReY);
		end
	end
end
colorCReY=[.6,.4,.2];
for indexPlotReY=1:length(PlotReY)
	plot(real(YNoIndex(indexPlotReY,:)),imag(YNoIndex(indexPlotReY,:)),'Color',colorCReY)
	hold on
	if indexPlotReY==1
		reYNoValues=real(YNoIndexLabel(indexPlotReY));
		imYNoValues=imag(YNoIndexLabel(indexPlotReY));
	else
		reYNoValues=horzcat(reYNoValues,real(YNoIndexLabel(indexPlotReY)));
		imYNoValues=horzcat(imYNoValues,imag(YNoIndexLabel(indexPlotReY)));
	end
	reYLabels=text(reYNoValues(indexPlotReY),imYNoValues(indexPlotReY),...
		[num2str(PlotReY(indexPlotReY),'%10.2f'),'  '],...
		'FontSize',9,'color',colorCReY);
	set(reYLabels,'Rotation',-90);
end

% Circles of Imaginary Y for Values of constant imaginary load impedances, B=const
ThetaRadCirImY=linspace(0,2*pi,NOP*4);
YNoIndexPosLabel(length(PlotImYpos))=0;
YNoIndexNegLabel(length(PlotImYneg))=0;
for trcImY=1:length(ThetaRadCirImY)
	for indexPlotImY=1:length(PlotImYpos)
		rp=PlotImYpos(indexPlotImY); % Diagramm-Radius, Reflexionsfaktor
		rn=PlotImYneg(indexPlotImY);
		% Imaginary Circle. Points G=x+jy
		YNoIndexPos(indexPlotImY,trcImY)=exp(j*pi)*((1+j*(1/rp))+(1/rp)*exp(j*ThetaRadCirImY(trcImY)));
		YNoIndexNeg(indexPlotImY,trcImY)=exp(j*pi)*((1+j*(1/rn))+(1/rn)*exp(j*ThetaRadCirImY(trcImY)));
		if trcImY==round(4*NOP/(1+0.60)) % for Labels
			YNoIndexPosLabel(indexPlotImY)=YNoIndexPos(indexPlotImY,trcImY);
		end
	end
end
colorCImY=[1,.6,.2];
for indexPlotImY=1:length(PlotImYpos)
	plot(real(YNoIndexPos(indexPlotImY,:)),imag(YNoIndexPos(indexPlotImY,:)),'Color',colorCImY)
	plot(real(YNoIndexNeg(indexPlotImY,:)),imag(YNoIndexNeg(indexPlotImY,:)),'Color',colorCImY)
	hold on
	if indexPlotImY==1
		reYposValues=real(YNoIndexPosLabel(indexPlotImY));
		imYposValues=imag(YNoIndexPosLabel(indexPlotImY));
	else
		reYposValues=horzcat(reYposValues,real(YNoIndexPosLabel(indexPlotImY)));
		imYposValues=horzcat(imYposValues,imag(YNoIndexPosLabel(indexPlotImY)));
	end
	YposLabels=text(reYposValues(indexPlotImY),imYposValues(indexPlotImY),...
		['j',num2str(abs(PlotImYpos(indexPlotImY)),'%10.2f')],...
		'FontSize',9,'color',colorCImY); % [sign(PlotImYpos(indexPlotImY)),'j',
	set(YposLabels,'Rotation',-90);
end
end % constructAdmittanceSmith==true

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Labels
ThetaColor=[.4,.4,1];
NOPTheta=37;
ThetaRadCirRhoLabels=linspace(0,2*pi,NOPTheta); % Angle
for ThetaIndex=1:(NOPTheta-1) % no 360?, but 0?
	ThetaPnt(ThetaIndex)=exp(j*ThetaRadCirRhoLabels(ThetaIndex));
	plot(real(ThetaPnt(ThetaIndex)),imag(ThetaPnt(ThetaIndex)),'b*'); hold on
	ThetaPnt(ThetaIndex)=0.97*exp(j*ThetaRadCirRhoLabels(ThetaIndex));
	ThetaLabels=text(real(ThetaPnt(ThetaIndex)),imag(ThetaPnt(ThetaIndex)),...
		[int2str(round(180/pi*ThetaRadCirRhoLabels(ThetaIndex))),''],...
		'FontSize',9,'color',ThetaColor);
	set(ThetaLabels,'Rotation',-90+180/pi*ThetaRadCirRhoLabels(ThetaIndex));
end

lDLambdaColor=[.6,.2,1];
lDLambda12=-linspace(-0.26,0,27);
NOPTheta=26;
ThetaRadCirRhoLabels=linspace(0,pi,NOPTheta); % Angle
for lDLindex=27:-1:2 % lDLambda12(27)=0 && lDLambda12(2)=0.25
	lDLambdaPnt(lDLindex)=exp(j*ThetaRadCirRhoLabels(lDLindex-1));
	plot(real(lDLambdaPnt(lDLindex)),imag(lDLambdaPnt(lDLindex)),'m*'); hold on
	lDLambdaPnt(lDLindex)=1.04*exp(j*ThetaRadCirRhoLabels(lDLindex-1));
	lDLambdaLabels=text(real(lDLambdaPnt(lDLindex)),imag(lDLambdaPnt(lDLindex)),...
		[num2str(lDLambda12(lDLindex),'%10.2f'),'  '],...
		'FontSize',9,'color',lDLambdaColor);
	set(lDLambdaLabels,'Rotation',-90+180/pi*ThetaRadCirRhoLabels(lDLindex-1));
end
lDLambda34=-linspace(-0.5,-0.26,25);
NOPTheta=26;
ThetaRadCirRhoLabels=linspace(pi,2*pi,NOPTheta); % Angle
for lDLindex=25:-1:2 % lDLambda34(25)=0.26 && lDLambda34(2)=0.49
	lDLambdaPnt(lDLindex)=exp(j*ThetaRadCirRhoLabels(lDLindex));
	plot(real(lDLambdaPnt(lDLindex)),imag(lDLambdaPnt(lDLindex)),'m*'); hold on
	lDLambdaPnt(lDLindex)=1.04*exp(j*ThetaRadCirRhoLabels(lDLindex));
	lDLambdaLabels=text(real(lDLambdaPnt(lDLindex)),imag(lDLambdaPnt(lDLindex)),...
		[num2str(lDLambda34(lDLindex),'%10.2f'),'  '],...
		'FontSize',9,'color',lDLambdaColor);
	set(lDLambdaLabels,'Rotation',-90+180/pi*ThetaRadCirRhoLabels(lDLindex));
end

set(gcf,'Color',[1,1,1]); % Hintergrundsfarbe
axis([-1.0 1.0 -1.0 1.0],'square','off'); % axis([Xmin Xmax Ymin Ymax],mode,visibility)
